/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package buildtools.java.packagebrowser;


import buildtools.*;
import java.util.zip.*;
import java.io.*;
import java.util.*;

public class PackageBrowser {
    
    public static final String PACKAGE_SEPARATOR = ".";

    private static boolean      parsed      = false;
    private static Hashtable    packages    = new Hashtable();
    
    
    /**
     * Return all known packages
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    public static JavaPackage[] getPackages() {

        parse();
        JavaPackage[] pkgs = new JavaPackage[packages.size()];
        
        Enumeration enum = packages.elements();
        int element = 0;
        while(enum.hasMoreElements() ) {
            pkgs[element] = (JavaPackage)enum.nextElement();
            ++element;
        }

        return pkgs;
    }
    
    /**
     * Parse out the current classpath and get all packages and classes.
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    public static void parse() {
        
        if (parsed == false) {
        
            String[] classpath = JavaUtils.getClasspath();
            for (int i = 0; i < classpath.length; ++i) {
                
                //StaticLogger.log( classpath[i] );
                File entry = new File( classpath[i] );
                if ( entry.isFile() ) {
                    parseJAR(entry);
                }
                
            }
        
            parsed = true;
        }
    }


    /**
     * Parse out a JAR file and added it to the known packages
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    private static void parseJAR(File jar) {


        try {
            //StaticLogger.log( "archive: " + jar.getAbsolutePath() );
            ZipFile archive = new ZipFile( jar.getAbsolutePath() );

            Enumeration elements = archive.entries();
    

            while( elements.hasMoreElements() ) {
                ZipEntry entry = (ZipEntry)elements.nextElement();


                /*
                if (jar.getName().equals("classes.zip") ) {
                    StaticLogger.log( entry.getName() );
                }
                */


                
                String ext = FileUtils.getExtension( entry.getName() );
    
                if ( ext != null && ext.equals("class") ) {
    
                    //if the entry is an inner class... ignore it.
                    if (entry.getName().indexOf("$") != -1) {
                        continue;
                    }
                    
                    String pkg = getPackage( entry.getName() );
    
                    if (pkg != null) {
                        String className = getFullyQualifiedClassName( entry.getName() );
    
                        getJavaPackage( pkg ).addClass( new JavaClass( className, jar.getAbsolutePath() ) );
                    }
    
                    //StaticLogger.log( getPackage( entry.getName() ) );
                }
                


            }


            
            

        } catch (IOException e) {
            e.printStackTrace();
            return;
        }


    }

    /**
     * Given a package name, return its JavaPackage from the stack or create a 
     * new one if necessary.
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    private static JavaPackage getJavaPackage(String packageName) {
        JavaPackage entry = (JavaPackage)packages.get(packageName);
        
        if (entry == null) {
            entry = new JavaPackage(packageName);
            packages.put(packageName, entry);
        }
        
        return entry;
    }
    
    /**
     * Given a classname... get its package. or null if it is not part of a package
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    private static String getPackage(String classname) {
        
        String ext = FileUtils.getExtension(classname);
        if ( ext != null && ext.equals("class") ) {

            int start = 0;
            int end = classname.lastIndexOf(System.getProperty("file.separator"));

            if (end == -1) {
                return null;
            }

            String rawpackage = classname.substring( start, end );
            
            
            return MiscUtils.globalStringReplace( rawpackage, System.getProperty("file.separator"), PACKAGE_SEPARATOR );
            
        } else {
            throw new IllegalArgumentException("this method only excepts .class files");
        }
        
    }

    /**
     * Given a file, return the full class path.
     *
     * EX:  org/apache/jetspeed/Test.class == org.apache.jetspeed.Test
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    private static String getFullyQualifiedClassName(String file) {

        int start = 0;
        int end = file.lastIndexOf(".class");
        
        if (end != -1) {
            file = file.substring( start, end );
        }
        return MiscUtils.globalStringReplace( file, 
                                              System.getProperty("file.separator"), 
                                              PACKAGE_SEPARATOR );
    }
    
    /**
     * dump all packages and classes
     * 
     * @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
     * @version $Id: PackageBrowser.java,v 1.3 2000/01/09 06:10:52 burton Exp $
     */
    public static void dump() {

        JavaPackage[] packages = getPackages();
        for (int i = 0;i < packages.length; ++i) {
            StaticLogger.log("PACKAGE -> " + packages[i].getName());
            packages[i].dump();

        }
        
    }
    
}

